/*
 * process, Distribution repositories and basic setup for Emmanuel Bigeon projects
 * Copyright (C) 2014-2017 E. Bigeon
 * mailto:emmanuel@bigeon.fr
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc-process:fr.bigeon.gclc.process.CommandFork.java
 * Created on: Nov 13, 2017
 */
package fr.bigeon.gclc.process;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import fr.bigeon.gclc.command.CommandParameters;
import fr.bigeon.gclc.command.ICommand;
import fr.bigeon.gclc.command.ICommandProvider;
import fr.bigeon.gclc.command.ParametrizedCommand;
import fr.bigeon.gclc.exception.CommandRunException;
import fr.bigeon.gclc.exception.CommandRunExceptionType;
import fr.bigeon.gclc.exception.InvalidParameterException;
import fr.bigeon.gclc.manager.ConsoleInput;
import fr.bigeon.gclc.manager.ConsoleOutput;

/** A command that is launched inside an internal terminal.
 * <p>
 * Several things are to be considered before adding this command to an
 * application:
 * <ul>
 * <li>The commands will be able to run in parallel.
 * <li>The managing of the commands will be handled through piped systems and
 * buffered in so switching to a forled command will actually print again all
 * the command history to the output console.
 * </ul>
 *
 * @author Emmanuel Bigeon */
public class CommandFork extends ParametrizedCommand {

    private final TaskPool pool = new TaskPool();
    private ICommandProvider provider;

    /** Add the fork command.
     *
     * @param name the command name */
    public CommandFork(final String name) {
        super(name);
        addParameters();
    }

    /**
     *
     */
    private void addParameters() {
        try {
            addStringParameter("join", false);
            addBooleanParameter("list");
        } catch (final InvalidParameterException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.ParametrizedCommand#doExecute(fr.bigeon.gclc.
     * manager.ConsoleOutput, fr.bigeon.gclc.manager.ConsoleInput,
     * fr.bigeon.gclc.command.CommandParameters) */
    @Override
    protected void doExecute(final ConsoleOutput out, final ConsoleInput in,
                             final CommandParameters parameters) throws CommandRunException {
        if (parameters.getBool("list")) {
            for (final String id : pool.getPIDs()) {
                try {
                    out.println(id + "\t" + pool.get(id).getName());
                } catch (final IOException e) {
                    throw new CommandRunException(
                            CommandRunExceptionType.INTERACTION,
                            "Failed to write to console", e, this);
                }
            }
        }

        final String string = parameters.get("join");
        if (string != null) {
            // Join the command.
            final ForkTask cmd = (ForkTask) pool.get(string);
            if (cmd == null) {
                throw new CommandRunException("No such fork process", this);
            }
            cmd.join(out, in);
            return;
        }

        final List<String> strings = parameters.getAdditionals();
        final ICommand cmd = provider.get(strings.get(0));
        final String[] args = Arrays.copyOfRange(strings.toArray(new String[0]),
                1, strings.size());
        final ForkTask task = new ForkTask(cmd, args);
        final Thread th = new Thread(task);
        pool.add(task);
        th.start();
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.ICommand#tip() */
    @Override
    public String tip() {
        // TODO Auto-generated method stub
        // return null;
        throw new RuntimeException("Not implemented yet");
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.Command#usageDetail() */
    @Override
    protected String usageDetail() {
        // TODO Auto-generated method stub
        // return null;
        throw new RuntimeException("Not implemented yet");
    }

}
